/*
 *  Copyright (C) 2012 Thomas Luka, 18 rue des feuilles mortes, 95 Vaureal
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef HELPWINDOW_H_INCLUDED
#define HELPWINDOW_H_INCLUDED
#include <QtGui>
#include <QtCore>
#include <QApplication>
#include "giac.h"
#include "History.h"

using namespace giac;

enum ChampCompleter
{
    ALL, DESCRIPTION, EXEMPLES, VOIRAUSSI
};

class HelpWindow : public QWidget
{
  Q_OBJECT

public:
    HelpWindow(QString cmd = "");
    HelpWindow(std::vector<aide> & helpVector, int CountHelp, QString cmd = "");
    /*Pour android afin d'éviter un chargement trop trop long
    on recuperera l'aide au demlarrage de l'appli,
    ATTENTION : AUCUNE COMMANDE N'EST TRIEE AVEC CE CONSTRUCTEUR
    LE TRIE DOIT ETRE EFFECTER AU PREALABLE*/

  aide RechercherCommande(QString Commande);
  void CompleterChamp(QString cmd, ChampCompleter valueCh = ALL);
  QStringList allHelpCmd(std::vector<aide> & VectorCmd);

public slots:
  void itemClicked(const QModelIndex & index);
  void RadioBouttonToggle(bool Checked);
  void OkClicked();
  void CmdClicked();
  void LineEditCmd();
  void LineEditModified(const QString NewText);//Pour selectionner les commandes correspondant au debut du text

  /*Slot for android interface*/
  void DescriptionWindow();
  void ExemplesWindow();
  void VoirAussiWindow();
  void VoirAussiClicked();
  void AnnulExempleClicked();

signals:
  void ExempleSelected(QString ex);
  void HelpClose();

private:

  void Language();//Pour initialiser la variable language
  void CreateAndroidWin();//Pour creer le fentre android et pouvoir l'utiliser dans les deux costructeur

  QString DescriptionCommande(const aide & cmd);//Pour avoir la description de la commande et la synthaxe
  QStringList ExemplesCommande(const aide & cmd);//Pour avoir les exemples
  QStringList Similitudes(const aide & cmd, bool syn = false);//Pour avoir les commandes simmilaires
  QStringList SyntaxCmdRessemblante(QString Cmd);//Pour savoir quel commande ressemble a Cmd

  QGroupBox *m_Description;
  QGroupBox *m_Exemples;
  QGroupBox *m_CommandesSimilaires;
  QVBoxLayout * m_LayoutExemple;
  QGridLayout * m_LayoutSimilaire;

  std::vector<aide> m_CasHelp;
  int m_CountHelp;
  QListView *m_CommandesView;
  QStandardItemModel *m_CommandesModel;
  QLineEdit *m_rechercher;
  int m_language;
  QLabel *m_DescriptionLabel;
  QString m_ExempleSelec;
};


#endif // HELPWINDOW_H_INCLUDED
