/*
 *  Copyright (C) 2012 Thomas Luka, 18 rue des feuilles mortes, 95 Vaureal
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "History.h"

History::~History(){
  delete contextptr;
}

//Class History
History::History() : QWidget(), m_thread(NULL)
{
  contextptr = new giac::context;
  m_cmdModel = new QStandardItemModel;
  m_cmdView = new QListView;
  m_cmdView->setModel(m_cmdModel);
  m_cmdView->setWordWrap(true);

  m_Copier = new QPushButton(tr("Copier"));
  m_Clear = new QPushButton(tr("Effacer l'historique"));

  m_LayoutWidget = new QVBoxLayout;
  m_LayoutWidget->addWidget(m_Copier);
  m_LayoutWidget->addWidget(m_cmdView);
  m_LayoutWidget->addWidget(m_Clear);
  setLayout(m_LayoutWidget);

  connect(m_cmdModel, SIGNAL(dataChanged(const QModelIndex, const QModelIndex)), this, SLOT(changeCmd(const QModelIndex, const QModelIndex)));
  connect(m_Copier, SIGNAL(clicked()), this, SLOT(CopierClicked()));
  connect(m_Clear, SIGNAL(clicked()), this, SLOT(ClearClicked()));
}
void History::addCmd(const QString Cmd)
{
  StandardItem *cmdItem = new StandardItem(true, Cmd);
  QString Ans = "   Calcul en attente";//On ajoute trois espaces pour dire que c'est une reponse et pas une commande
  StandardItem * ansItem= new StandardItem(false, Ans);
  m_cmdModel->insertRow(0, ansItem);//pour qu'il soit en dessous de la commande
  m_cmdModel->insertRow(0, cmdItem);
#if 0
  m_cmdModel->setData(ansItem->index(), Qt::AlignRight, Qt::TextAlignmentRole);//Pour que le text soit aligner a droite
#endif

  cmdItem->setAnsStandardItem(ansItem);

  if (m_thread == NULL)
  {
      m_a_calculer.push_back(cmdItem);
      Evaluate();
  }
  else
  {
      m_a_calculer.push_back(cmdItem);
      if (!m_thread->isRunning())
          Evaluate();
  }
  m_cmdView->setCurrentIndex(ansItem->index());
}
void History::changeCmd(const QModelIndex &index, const QModelIndex &indexDeux)
{
  StandardItem *cmd = static_cast<StandardItem *>(m_cmdModel->itemFromIndex(index));//On recupere l'item
  if (cmd->isCmd())
    {
      m_a_calculer.push_back(cmd);
      if (m_a_calculer.size() == 1
              && !m_thread->isRunning())
          Evaluate();
      cmd->ansStandardItem()->setData("   Calcul en attente", Qt::DisplayRole);//On remet les donnes
    }
}
void History::CopierClicked()
{
  QModelIndex itemIndex;
  QItemSelection select = m_cmdView->selectionModel()->selection();
  QModelIndexList ModelSelect = select.indexes();
  if (!ModelSelect.isEmpty())
    itemIndex = ModelSelect.at(0);//Une selection autoriser donc un element dans la liste si elle n'est pas vide
  QString strCp = m_cmdModel->data(itemIndex, Qt::DisplayRole).toString();
  strCp.remove(" ");
  emit strCpSignal(strCp);
}
void History::ClearClicked()
{
  m_cmdModel->clear();
}
void History::Evaluate()
{
  /*std::string S = Expr.toStdString();//On le convertit en std::string
  try {
    gen g(S,contextptr);//On cree un objet gen
    g = eval(g,1,contextptr);//on evalue l'expression
    if (g.is_symb_of_sommet(at_pnt))
    {
        QWidget *widget = new QWidget;
        QVBoxLayout *vbox = new QVBoxLayout(widget);
        QPushButton *ok = new QPushButton("Ok");
        GraphWidget *graph = new GraphWidget(g, contextptr, false);
        vbox->addWidget(graph);
        vbox->addWidget(ok);
        widget->setLayout(vbox);
        widget->show();
        connect(ok, SIGNAL(clicked()), widget, SLOT(deleteLater()));
    }
    S = g.print(contextptr);//On recupere le resultat
  }
  catch (std::runtime_error & err){
    S=err.what();
  }
  return QString::fromStdString(S);*/
    m_traitement = m_a_calculer.takeFirst();
    if (m_traitement == NULL)
        return;
    m_traitement->ansStandardItem()->setData("   Calcul en cours", Qt::DisplayRole);
    std::string S = m_traitement->data(Qt::DisplayRole).toString().toStdString();
    gen g(S, contextptr);
    m_thread = new EvalThread(g,contextptr, this);
    m_thread->start();
    emit calcul_en_cours(true);
    connect(m_thread, SIGNAL(calcul_finished()), this, SLOT(EvalFinished()));
    connect(m_thread, SIGNAL(error(QString)), this, SLOT(EvalError(QString)));
    connect(m_thread, SIGNAL(finished()), this, SLOT(suite_calcul()));
}
QStringList History::allCmds()
{
    QStringList listItems;
    QList<QStandardItem *> items = m_cmdModel->takeColumn(0);
    for (int i = 0; i < items.size(); i++)
    {
        StandardItem *stdItem = static_cast<StandardItem *>(items.at(i));
        if (stdItem->isAns())
            listItems.push_back("0:"+stdItem->text());
        if (!stdItem->isAns())
            listItems.push_back("1:"+stdItem->text());
    }
    return listItems;
}
void History::setAllItems(QStringList items)
{
    StandardItem *cmdItem;
    for(int i = 0; i < items.size(); i++)
    {
        QString string = items.at(i);
        string.remove('\n');
        if (string.left(2) == "1:")
            cmdItem = addItem(string.right(string.size()-2), true);
        else
            cmdItem->setAnsStandardItem(addItem(string.right(string.size()-2), false));
    }
}
void History::EvalError(QString s)
{
    //QMessageBox::information(0,"","Erreur calcul");
    m_thread->terminate();
    m_traitement->ansStandardItem()->setData("   "+s, Qt::DisplayRole);
    emit calcul_en_cours(false);
    giac::ctrl_c = false;
}
void History::EvalFinished()
{
    gen g = static_cast<EvalThread *>(QObject::sender())->getGen();
    //QMessageBox::information(0,"","Finished");
    if (g.is_symb_of_sommet(at_pnt))
    {
        QWidget *widget = new QWidget;
        QVBoxLayout *vbox = new QVBoxLayout(widget);
        QPushButton *ok = new QPushButton("Ok");
        GraphWidget *graph = new GraphWidget(g, contextptr, false);
        vbox->addWidget(graph);
        vbox->addWidget(ok);
        widget->setLayout(vbox);
        widget->show();
        connect(ok, SIGNAL(clicked()), widget, SLOT(deleteLater()));
    }
    m_traitement->ansStandardItem()->setData("   "+QString::fromStdString(g.print(contextptr)), Qt::DisplayRole);
}
void History::stop_calcul()
{
    giac::ctrl_c = true;
    QTimer *timer = new QTimer;
    timer->start(2000);
    connect(timer, SIGNAL(timeout()), this, SLOT(terminate_thread()));
    connect(timer, SIGNAL(timeout()), timer, SLOT(deleteLater()));
}
void History::suite_calcul()
{
    if (!m_a_calculer.isEmpty())
        Evaluate();
    else
    {
        m_thread->deleteLater();
        m_thread = NULL;
    }
}
void History::terminate_thread()
{
    if (m_thread != NULL)
    {
        m_thread->terminate();
        EvalError("Interruption de l'utilisateur");
        suite_calcul();
    }
}

StandardItem* History::addItem(QString text, bool cmd)
{
    StandardItem *item = new StandardItem(cmd, text);
    m_cmdModel->appendRow(item);
    return item;
}
//class StandardItem
StandardItem::StandardItem(bool cmd) : QStandardItem()
{
  m_ans = NULL;
  m_isCmd = cmd;
  if (!cmd)
      setEditable(false);
}
StandardItem::StandardItem(bool cmd, const QString & text) : QStandardItem(text)
{
  m_isCmd = cmd;
  m_ans = NULL;
  if (!cmd)
      setEditable(false);
}
StandardItem::StandardItem(bool cmd,const QString & text, StandardItem *ans) : QStandardItem(text)
{
  m_isCmd = cmd;
  m_ans = ans;
  if (!cmd)
      setEditable(false);
}
void StandardItem::setAnsStandardItem(StandardItem *ans)
{
  m_ans = ans;
}
StandardItem * StandardItem::ansStandardItem()
{
  return m_ans;
}
bool StandardItem::isAns()
{
  if (m_ans == NULL)
    return true;

  return false;
}
bool StandardItem::isCmd()
{
  if (m_ans == NULL)
    return false;

  return true;
}
